#include "grbc/ext.h"
#include "grbc/helpers.h"
#include "grbc/spec.h"
#include "grbc/state.h"

void grbc_register_ext(const Extension &ext) {
  GState::get().extensions.push_back(ext);

  log_msg(("registered extension: " + ext.name).c_str());
  printf(">  init pointer: %p\n", ext.hook_init);
}

bool grbc_has_ext(const std::string &extension_id) {
  for (auto &ext : GState::get().extensions) {
    if (ext.name == extension_id)
      return true;
  }

  return false;
}

void grbc_ext(const std::string &extension_id) {
  for (auto &ext : GState::get().extensions) {
    if (ext.name != extension_id)
      continue;

    log_msg(("loading extension: " + extension_id).c_str());

    ext.hook_init(GState::get().lua);

    ext.loaded = true;

    return;
  }

  grbc_exception("Failed to find extension with name: " + extension_id);
}

bool grbc_is_ext_loaded(const std::string &extension_name) {

  for (auto &ext : GState::get().extensions) {
    if (ext.name == extension_name && ext.loaded)
      return true;
  }

  return false;
}

void grbc_ext_process_file(const std::string &file) {
  for (auto &ext : GState::get().extensions) {
    if (!ext.loaded || ext.process_file == nullptr)
      continue;

    ext.process_file(file);
  }
}